SystemOrganization addCategory: #'Phantom-Utility'!

PhAspect subclass: #PhPerObjDeployAspect
	instanceVariableNames: 'targetmethod classpattern aspectclass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Utility'!

!PhPerObjDeployAspect commentStamp: 'johanfabry 10/7/2011 16:13' prior: 0!
A deployment aspect that instantiates and deploys one aspect for each instantiation of an object of the class pattern. The aspect instance is sent the targetmethod message with argument the object just instantiated so it can keep track of it.

See PhPerObjExampleAspect for an example!

!PhPerObjDeployAspect classSide methodsFor: 'instance creation' stamp: 'johanfabry 10/7/2011 16:22'!
onClassPattern: aClassPat instantiate: anAspectClass targetmethod: aSymbol
	|asp|
	asp := self basicNew.
	asp classpattern: aClassPat.
	asp aspectclass:  anAspectClass.
	asp targetmethod: aSymbol.  
	asp initialize.
	^asp.! !

!PhPerObjDeployAspect methodsFor: 'accessing' stamp: 'johanfabry 10/7/2011 15:21'!
aspectclass
	^ aspectclass! !

!PhPerObjDeployAspect methodsFor: 'accessing' stamp: 'johanfabry 10/7/2011 15:21'!
aspectclass: anObject
	aspectclass := anObject! !

!PhPerObjDeployAspect methodsFor: 'accessing' stamp: 'johanfabry 10/7/2011 15:21'!
classpattern
	^ classpattern! !

!PhPerObjDeployAspect methodsFor: 'accessing' stamp: 'johanfabry 10/7/2011 15:21'!
classpattern: anObject
	classpattern := anObject! !

!PhPerObjDeployAspect methodsFor: 'initialize-release' stamp: 'johanfabry 10/7/2011 16:21'!
initialize
	super initialize.
	self add:
		(PhAdvice around: (PhPointcut receivers: self classpattern selectors: #new context: #(proceed) )
			send: #instantiate: to: self).
	self install.! !

!PhPerObjDeployAspect methodsFor: 'installing' stamp: 'johanfabry 10/7/2011 15:54'!
instantiate: aContext
	| theObject theAspect|
	theObject := aContext proceed.
	theAspect := self aspectclass new.
	theAspect perform: self targetmethod with: theObject.
	theAspect init.  
	^theObject.
	! !

!PhPerObjDeployAspect methodsFor: 'accessing' stamp: 'johanfabry 10/7/2011 15:27'!
targetmethod
	^ targetmethod! !

!PhPerObjDeployAspect methodsFor: 'accessing' stamp: 'johanfabry 10/7/2011 15:27'!
targetmethod: anObject
	targetmethod := anObject! !

PhAspect subclass: #PhPerObjExampleAspect
	instanceVariableNames: 'target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phantom-Utility'!

!PhPerObjExampleAspect commentStamp: 'johanfabry 10/7/2011 16:10' prior: 0!
An example of per-object deployment. The following produces the array { 1 . 1 . 0}

| dep m1 m2 m3 |
dep := PhPerObjDeployAspect onClassPattern: 'PhMockClass class' instantiate: PhPerObjExampleAspect targetmethod: #target:.
m1 := PhMockClass new.
m2 := PhMockClass new.
dep uninstall.
m3 := PhMockClass new.
m1 plus.
{(m1 counter) . (m2 counter) . (m3 counter)} inspect
!

!PhPerObjExampleAspect methodsFor: 'initialize-release' stamp: 'johanfabry 10/7/2011 16:03'!
init
	|plus|
	plus := PhPointcut receivers: 'PhMockClass' selectors: 'plus' context: #(receiver).

	self add: (PhAdvice after: plus
		advice: [:ctx | (ctx receiver = self target) ifFalse: [self target counter: (self target counter +1) ]] ).
	self install. ! !

!PhPerObjExampleAspect methodsFor: 'accessing' stamp: 'johanfabry 10/7/2011 15:37'!
target
	^ target! !

!PhPerObjExampleAspect methodsFor: 'accessing' stamp: 'johanfabry 10/7/2011 15:37'!
target: anObject
	target := anObject! !
